#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const argparse_1 = require("argparse");
const fs = __importStar(require("fs"));
const path = __importStar(require("path"));
const config = __importStar(require("./config"));
const Keystore_1 = require("./Keystore");
const chat = __importStar(require("./subcommands/chat"));
const addkey = __importStar(require("./subcommands/addkey"));
const removekey = __importStar(require("./subcommands/removekey"));
const showkey = __importStar(require("./subcommands/showkey"));
const genkey = __importStar(require("./subcommands/genkey"));
const init_1 = require("./ui/init");
const utils_1 = require("./utils");
function parseArgs() {
    const pkgBuff = fs.readFileSync(path.resolve(__dirname, '..', 'package.json'));
    const pkgJSON = JSON.parse(pkgBuff.toString('utf8'));
    const parser = new argparse_1.ArgumentParser({
        prog: pkgJSON.name,
        version: pkgJSON.version,
        description: pkgJSON.description
    });
    parser.addArgument(['--config', '-c'], {
        help: `Path to config file (default: ${config.defaultConfigPath})`,
        defaultValue: config.defaultConfigPath
    });
    const subs = parser.addSubparsers({
        title: 'subcommands',
        dest: 'subcommand'
    });
    const chat = subs.addParser('chat', {
        addHelp: true,
        description: 'Enter the chat room.'
    });
    chat; // intentionally unused
    const showKey = subs.addParser('showkey', {
        addHelp: true,
        description: 'List keys.'
    });
    showKey.addArgument('callsign', { type: 'string', nargs: '?' });
    const addKey = subs.addParser('addkey', {
        addHelp: true,
        description: 'Add a new public key to the keystore associated with a callsign.'
    });
    addKey.addArgument('callsign', { type: 'string' });
    addKey.addArgument('publickey', { type: 'string' });
    const removeKey = subs.addParser('removekey', {
        addHelp: true,
        description: 'Remove a public key from the keystore.'
    });
    removeKey.addArgument('callsign', { type: 'string' });
    removeKey.addArgument('publickey', { type: 'string' });
    const genKey = subs.addParser('genkey', {
        addHelp: true,
        description: 'Generate a new keypair for your callsign.'
    });
    genKey.addArgument('--make-signing', {
        action: 'storeTrue',
        dest: 'makeSigning',
        help: 'Make the generated key your default signing key.'
    });
    return parser.parseArgs();
}
function validateArgs(args) {
    if (args.callsign != null && !utils_1.isCallsign(args.callsign)) {
        console.error(`${args.callsign} is not a valid callsign.`);
        if (args.callsign.includes('-')) {
            console.error(`callsign should not include an SSID for key management subcommands.`);
        }
        process.exit(1);
    }
}
// not sure if we should add this here...
// function validateKeystoreFile(conf: config.Config): void {
//     if (!fs.existsSync(conf.keystoreFile)) {
//         console.error(`No keystoreFile exists at location "${conf.keystoreFile}".`)
//         process.exit(1)
//     } else {
//         try {
//             JSON.parse(fs.readFileSync(conf.keystoreFile).toString('utf8'))
//         } catch (err) {
//             console.error(`Error loading keystoreFile file from "${conf.keystoreFile}".`)
//             console.error(err.message)
//             process.exit(1)
//         }
//     }
// }
function validateSigningKeyExists(conf, ks) {
    // if there is a signing in the config but it doesn't exist in the keystore
    if (conf.signingKey != null) {
        const signing = ks.getKeyPairs(conf.callsign).filter((key) => {
            return key.public === conf.signingKey;
        });
        if (signing.length < 1) {
            console.error(`Default signing key has no matching private key found in the keystore.`);
            process.exit(1);
        }
    }
}
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        const args = parseArgs();
        validateArgs(args);
        // initialize a new config
        if (!config.exists(args.config)) {
            // if the default config doesn't exist, let's run the interactive init
            if (args.config === config.defaultConfigPath) {
                yield init_1.interactiveInit();
            }
            else {
                console.error(`No config file exists at "${args.config}".`);
                process.exit(1);
            }
        }
        let conf = null;
        try {
            conf = config.load(args.config);
        }
        catch (err) {
            console.error(`Error loading config file from "${args.config}".`);
            console.error(err.message);
            process.exit(1);
        }
        // validate that keystore file exists
        // validateKeystoreFile(conf)
        const ks = new Keystore_1.Keystore(conf.keystoreFile);
        // if this subcommand is any of the commands that signs something
        if (['chat'].includes(args.subcommand)) {
            validateSigningKeyExists(conf, ks);
        }
        let code = null;
        switch (args.subcommand) {
            case 'chat':
                code = yield chat.main(args, conf, ks);
                break;
            case 'showkey':
                code = yield showkey.main(args, conf, ks);
                break;
            case 'addkey':
                code = yield addkey.main(args, conf, ks);
                break;
            case 'removekey':
                code = yield removekey.main(args, conf, ks);
                break;
            case 'genkey':
                code = yield genkey.main(args, conf, ks);
                break;
        }
        process.exit(code);
    });
}
main()
    .catch((err) => {
    console.error(err);
    process.exit(1);
});
//# sourceMappingURL=main.js.map